/******************************************************************************
*               COPYRIGHT (C) 2009 by RSA Security Inc.                  *
*                         ---ALL RIGHTS RESERVED---                           *
*                                                                             *
* THIS SOFTWARE IS PROPRIETARY AND CONFIDENTIAL TO RSA Security Inc., IS      *
* FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED ONLY IN ACCORDANCE THE *
* TERMS OF SUCH LICENSE AND WITH THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE. *
* THIS SOFTWARE OR ANY OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE  *
* MADE AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE       *
* SOFTWARE IS HEREBY TRANSFERRED.                                             *
*                                                                             *
* THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND    *
* SHOULD NOT BE CONSTRUED AS A COMMITMENT BY RSA Security Inc.                *
******************************************************************************/

//rsacookieapi.h -- rsacookieapi shared object entry points.

//The following ifdef block is the standard way of creating macros 
//which make exporting from a DLL simpler. All files within this DLL 
//are compiled with the RSACOOKIEAPI_EXPORTS symbol defined on the 
//command line. this symbol should not be defined on any project that 
//uses this DLL. This way any other project whose source files 
//include this file see RSACOOKIEAPI_API functions as being imported 
//from a DLL, wheras this DLL sees symbols defined with this macro 
//as being exported.

#ifdef WIN32
#include <HttpFilt.h>
#ifdef RSACOOKIEAPI_EXPORTS
#define RSACOOKIEAPI_API __declspec(dllexport)
#else
#define RSACOOKIEAPI_API __declspec(dllimport)
#endif
#else
#define RSACOOKIEAPI_API
#define WINAPI

//wrap the typedef to avoid multiple type defines
#ifndef TYPEDEF_LPCSTR
#define TYPEDEF_LPCSTR
typedef const char * LPCSTR;
#endif


#ifdef WIN32
#ifndef TYPEDEF_SD_BOOL
#define TYPEDEF_SD_BOOL
typedef signed long int SD_BOOL;
#endif
#endif


#endif


// return codes from the API
// when the return is NULL GetLastError() will get one of these
///
#define RSACOOKIE_ERROR_NO_ERROR                    0
#define RSACOOKIE_ERROR_CANNOT_ACCESS_SETTINGS      100
#define RSACOOKIE_ERROR_VALID_COOKIE_NOT_FOUND      101
#define RSACOOKIE_ERROR_DATA_TAG_NOT_FOUND          102
#define RSACOOKIE_ERROR_NOT_ENOUGH_MEMORY           103
#define RSACOOKIE_ERROR_INVALID_ARGUMENT            104
#define RSACOOKIE_ERROR_CIPHERSUITE_ERROR           105   
#define RSACOOKIE_ERROR_LONGDATALEN_ENCRYPTION      106
#define RSACOOKIE_ERROR_TOOMANY_ENCRYPTED_FIELDS    107

#ifndef MOD_DEBUG
#define MOD_DEBUG     0x00000009L
#endif
#ifdef __cplusplus__
extern "C" {
#endif

#ifdef WIN32
    // This function needs to be declared so that our ASP object can call it 
    // from onstartpage. It is not exposed as part of the dll though.
    extern int errInitSettingsFromHeader(const char *settings);
  
    // Shared object entry points
    
    // This initialization fnction is to be called from C programs 
    // to initialize the cookie api
    RSACOOKIEAPI_API int WINAPI RSACookieInitializeCGI();
    
    // This initialization function is to be called from  isapi filters
    // to initialize the cookie api. RSA filters need to be called first
    // before you make a call to this.
    RSACOOKIEAPI_API int WINAPI RSACookieInitializeFilt(HTTP_FILTER_CONTEXT* pCtxt);

//This define allows simple cgi scripts to avoid linking with MFC
#ifndef COOKIE_API_NOISAPI
    // This initialization function is to be called from  isapi extensions
    // for MFC based applications to initialize the cookie api
    RSACOOKIEAPI_API int WINAPI RSACookieInitializeExt(CHttpServerContext *pCtxt);
    
    // This initialization function is to be called from  isapi extensions
    // for non-MFc based apps to initialize the cookie api
    RSACOOKIEAPI_API int WINAPI RSACookieInitializeExtNoMFC(EXTENSION_CONTROL_BLOCK* pCtxt);
#endif
#endif  
    // NOTE: all these functions return allocated data which must
    // be freed using RSAFreeMemory() when the caller is done.

	//Will return WebID URL path,
    // must be "freed" by the caller. 
	RSACOOKIEAPI_API LPCSTR WINAPI RSAGetWebIDURL(LPCSTR szID);
													
    // Will return allocated buffer with "Shell" field contents,
    // must be "freed" by the caller. If the cookie is invalid or
    // missing it will return NULL.


    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetShellField(LPCSTR szID,
                                                    LPCSTR Cookie,
                                                    LPCSTR User,
                                                    LPCSTR BrowserIP,
                                                    LPCSTR Agent);

    // Will return allocated buffer with "Session" field contents,
    // must be "freed" by the caller. If the cookie is invalid or
    // missing it will return NULL.
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetCSRFToken(LPCSTR szID,
                                                    LPCSTR Cookie,
                                                    LPCSTR User,
                                                    LPCSTR BrowserIP,
                                                    LPCSTR Agent);

    // Will return allocated buffer with "User" field contents,
    // must be "freed" by the caller. If the cookie is invalid or
    // missing it will return NULL.
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetUserName(LPCSTR szID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent);

    // Will return allocated buffer with "Tag" field contents,
    // must be "freed" by the caller. If the cookie is invalid or
    // missing or the "Tag" field is not set it will return NULL.
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetTagField(LPCSTR szID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent,
                                                  LPCSTR Tag,
                                                  SD_BOOL Encrypted);

    // Will return allocated buffer with new cookie headers,
    // must be "freed" by the caller.
    RSACOOKIEAPI_API LPCSTR WINAPI RSASetTagField(LPCSTR szID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent,
                                                  LPCSTR Tag,
                                                  LPCSTR Data,
                                                  SD_BOOL Encrypted);

    // Will return allocated buffer with new cookie headers,
    // must be "freed" by the caller.
    RSACOOKIEAPI_API LPCSTR WINAPI RSADeleteTagField(LPCSTR szID,
                                                     LPCSTR Cookie,
                                                     LPCSTR User,
                                                     LPCSTR BrowserIP,
                                                     LPCSTR Agent,
                                                     LPCSTR Tag);

    // This function will free the allocated data returned by
    // all of the functions listed above
    RSACOOKIEAPI_API void WINAPI RSAFreeMemory(LPCSTR Buffer);

    // Function used to get the errno on unix and to call the
    // NT GetLastError()
    RSACOOKIEAPI_API int WINAPI RSAGetLastError (void);
#ifdef WIN32
    // Wrapper functions to maintain backward compatibility on windows
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetWebIDURL(INT ID);
	
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetUserName(INT ID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent);
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetShellField(INT ID,
                                                    LPCSTR Cookie,
                                                    LPCSTR User,
                                                    LPCSTR BrowserIP,
                                                    LPCSTR Agent);
    RSACOOKIEAPI_API LPCSTR  WINAPI RSAGetCSRFToken(INT ID,
                                                    LPCSTR Cookie,
                                                    LPCSTR User,
                                                    LPCSTR BrowserIP,
                                                    LPCSTR Agent);
    RSACOOKIEAPI_API LPCSTR WINAPI RSADeleteTagField(INT ID,
                                                     LPCSTR Cookie,
                                                     LPCSTR User,
                                                     LPCSTR BrowserIP,
                                                     LPCSTR Agent,
                                                     LPCSTR Tag);
     RSACOOKIEAPI_API LPCSTR WINAPI RSASetTagField(INT ID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent,
                                                  LPCSTR Tag,
                                                  LPCSTR Data,
                                                  SD_BOOL Encrypted);
    RSACOOKIEAPI_API LPCSTR WINAPI RSAGetTagField(INT ID,
                                                  LPCSTR Cookie,
                                                  LPCSTR User,
                                                  LPCSTR BrowserIP,
                                                  LPCSTR Agent,
                                                  LPCSTR Tag,
                                                  SD_BOOL Encrypted);


#endif

#ifdef __cplusplus__
}
#endif

